#include "LoaderUtils.hpp"

#include <cassert>
#include <util/Logger.hpp>
#include <util/String.hpp>

std::pair< std::string, std::string > LoaderUtils::parseSides( const std::string& str, std::string sep )
{
	std::size_t pos = str.find( sep );
	
	std::string left = str.substr( 0, pos );
	std::string right = str.substr( pos + sep.length() );
	
	return std::make_pair( left, right );
}

LoaderUtils::Parsed< sf::Vector2i > LoaderUtils::parseVec2( const std::string& str )
{
	assert( str[ 0 ] == '(' );
	std::size_t end = str.find( ')' );
	assert( end != std::string::npos );
	
	std::string main = str.substr( 1, end - 1 );
	std::string leftover = str.substr( end + 1 );
	
	std::pair< std::string, std::string > nums = parseSides( main, "," );
	int x = util::fromString< int >( nums.first );
	int y = util::fromString< int >( nums.second );
	
	Parsed< sf::Vector2i > parsed;
	parsed.data = sf::Vector2i( x, y );
	parsed.leftover = leftover;
	
	return parsed;
}

LoaderUtils::Parsed< sf::IntRect > LoaderUtils::parseRect( const std::string& str )
{
	assert( str[ 0 ] == '[' );
	std::size_t end = str.find( ']' );
	assert( end != std::string::npos );
	
	std::string main = str.substr( 1, end - 1 );
	std::string leftover = str.substr( end + 1 );
	
	std::vector< std::string > tokens = util::tokenize( main, "," );
	int x = util::fromString< int >( tokens[ 0 ] );
	int y = util::fromString< int >( tokens[ 1 ] );
	int w = util::fromString< int >( tokens[ 2 ] );
	int h = util::fromString< int >( tokens[ 3 ] );
	
	Parsed< sf::IntRect > parsed;
	parsed.data = sf::IntRect( x, y, w, h );
	parsed.leftover = leftover;
	
	return parsed;
}
