#include "Options.hpp"

#include <SFML/Audio/Listener.hpp>
#include <util/IniFile.hpp>
#include <util/Logger.hpp>

Options::Options()
{
	// Add controls here, such as:
	controls.push_back( std::make_pair( "Pause", InputBinding( sf::Keyboard::Escape ) ) );
	controls.push_back( std::make_pair( "Up", InputBinding( sf::Keyboard::W ) ) );
	controls.push_back( std::make_pair( "Down", InputBinding( sf::Keyboard::S ) ) );
	controls.push_back( std::make_pair( "Left", InputBinding( sf::Keyboard::A ) ) );
	controls.push_back( std::make_pair( "Right", InputBinding( sf::Keyboard::D ) ) );
	controls.push_back( std::make_pair( "Action", InputBinding( sf::Keyboard::Space ) ) );
	controls.push_back( std::make_pair( "Drop Item", InputBinding( sf::Keyboard::R ) ) );
	controls.push_back( std::make_pair( "Slot 1", InputBinding( sf::Keyboard::Num1 ) ) );
	controls.push_back( std::make_pair( "Slot 2", InputBinding( sf::Keyboard::Num2 ) ) );
	controls.push_back( std::make_pair( "Slot 3", InputBinding( sf::Keyboard::Num3 ) ) );
	controls.push_back( std::make_pair( "Slot 4", InputBinding( sf::Keyboard::Num4 ) ) );
	controls.push_back( std::make_pair( "Slot 5", InputBinding( sf::Keyboard::Num5 ) ) );
	controls.push_back( std::make_pair( "Prev Item", InputBinding( sf::Keyboard::Q ) ) );
	controls.push_back( std::make_pair( "Next Item", InputBinding( sf::Keyboard::E ) ) );

	hardUnlocked = false;
}

void Options::save( const std::string& filename )
{
	util::IniFile config;
	config[ "General" ][ "Volume" ] = util::toString( sf::Listener::getGlobalVolume() );
	config[ "General" ][ "Hard Unlocked" ] = hardUnlocked ? "true" : "false";
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		config[ "Controls" ][ it->first ] = it->second.write();
	}

	config.saveToFile( filename );
}

void Options::load( const std::string& filename )
{
	util::IniFile config;
	if ( !config.loadFromFile( filename ) )
	{
		util::log( "Failed to load config file." );
		return;
	}

	sf::Listener::setGlobalVolume( util::fromString< float >( config[ "General" ][ "Volume" ] ) );
	if ( config[ "General" ].find( "Hard Unlocked" ) != config[ "General" ].end() )
	{
		hardUnlocked = ( config[ "General" ][ "Hard Unlocked" ] == "true" );
	}
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		if ( config[ "Controls" ].find( it->first ) == config[ "Controls" ].end() ) continue;
		it->second = InputBinding::read( config[ "Controls" ][ it->first ] );
	}
}

const InputBinding& Options::getInputBinding( const std::string& str ) const
{
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		if ( it->first == str )
		{
			return it->second;
		}
	}

	throw std::invalid_argument( "'" + str + "' is not an existing control." );
}

InputBinding& Options::getInputBinding( const std::string& str )
{
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		if ( it->first == str )
		{
			return it->second;
		}
	}

	throw std::invalid_argument( "'" + str + "' is not an existing control." );
}

std::vector< std::string > Options::getControlList() const
{
	std::vector< std::string > controlList;
	controlList.reserve( controls.size() );
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		controlList.push_back( it->first );
	}

	return controlList;
}

void Options::updateInput()
{
	for ( auto it = controls.begin(); it != controls.end(); ++it )
	{
		it->second.update();
	}
}
