#include "game/Area.hpp"

#include <cassert>
#include <fstream>
#include <SFML/Graphics/Image.hpp>
#include <util/Logger.hpp>

#include "LoaderUtils.hpp"

Area::Area( Game& theGame, Player& thePlayer, const std::string& theName, sf::Vector2i theSize )
   : game( theGame ),
     player( thePlayer ),
     name( theName ),
     size( theSize )
{
	util::log( "Creating area \"%s\", size (%i,%i)...\n", name, size.x, size.y );
	assert( size.x % 2 == 1 && size.y % 2 == 1 );

	sf::Image areaImage;
	areaImage.loadFromFile( "res/" + name + ".png" );

	rooms.reserve( size.x * size.y );
	for ( int iy = getMinimumRoom().y; iy <= getMaximumRoom().y; ++iy )
	{
		for ( int ix = getMinimumRoom().x; ix <= getMaximumRoom().x; ++ix )
		{
			rooms.push_back( std::make_shared< Room >( ( * this ), sf::Vector2i( ix, iy ) ) );

			// Cutting the main image into pieces
			sf::Vector2i pos;
			pos.x = ( ix + std::abs( getMinimumRoom().x ) ) * 30;
			pos.y = ( iy + std::abs( getMinimumRoom().y ) ) * 30;

			sf::Image image;
			image.create( 30, 30 );
			image.copy( areaImage, 0, 0, sf::IntRect( pos.x, pos.y, 30, 30 ) );

			image.saveToFile( "res/" + name + "/" + util::toString( ix ) + "_" + util::toString( iy ) + ".png" );
		}
	}

	std::ifstream file( "res/" + name + "/info.txt", std::ifstream::in );
	while ( file )
	{
		std::string line;
		std::getline( file, line );
		if ( file and line.length() >= 19 and line[ 0 ] != '#' )
		{
			std::pair< std::string, std::string > parts = LoaderUtils::parseSides( line );
			LoaderUtils::Parsed< sf::Vector2i > roomNum = LoaderUtils::parseVec2( parts.first );
			LoaderUtils::Parsed< sf::IntRect > box = LoaderUtils::parseRect( roomNum.leftover );

			util::log( "\tIn room (%i,%i) box (%i,%i,%i,%i) does %s.\n", roomNum.data.x, roomNum.data.y, box.data.left, box.data.top, box.data.width, box.data.height, parts.second );
			Room& room = getRoom( roomNum.data );
			std::string func = parts.second.substr( 0, parts.second.find( '(' ) );

			std::string paramStr =  parts.second.substr( func.length() + 1, parts.second.length() - func.length() - 2 );
			std::vector< std::string > params;
			std::size_t parenCount = 0;
			for ( std::size_t i = 0; i < paramStr.length(); ++i )
			{
				if ( paramStr[ i ] == ',' and parenCount == 0 )
				{
					params.push_back( paramStr.substr( 0, i ) );
					paramStr = paramStr.substr( i + 1 );
					i = -1;
					continue;
				}
				else if ( paramStr[ i ] == '(' )
				{
					++parenCount;
				}
				else if ( paramStr[ i ] == ')' )
				{
					--parenCount;
				}
			}
			params.push_back( paramStr );

			room.addBox( box.data, func, params );
		}
	}
}

sf::Vector2i Area::getMinimumRoom() const
{
	return sf::Vector2i( -( size.x - 1 ) / 2, -( size.y - 1 ) / 2 );
}

sf::Vector2i Area::getMaximumRoom() const
{
	return sf::Vector2i( ( size.x - 1 ) / 2, ( size.y - 1 ) / 2 );
}

sf::Vector2i Area::getSize() const
{
	return size;
}

Room& Area::getRoom( sf::Vector2i room )
{
	room.x += ( size.x / 2 );
	room.y += ( size.y / 2 );

	return ( * rooms[ room.x + ( room.y * size.x ) ] );
}

const Room& Area::getRoom( sf::Vector2i room ) const
{
	room.x += ( size.x / 2 );
	room.y += ( size.y / 2 );

	return ( * rooms[ room.x + ( room.y * size.x ) ] );
}

Player& Area::getPlayer()
{
	return player;
}

const Player& Area::getPlayer() const
{
	return player;
}

Game& Area::getGame()
{
	return game;
}

const Game& Area::getGame() const
{
	return game;
}
