#include "game/BombEntity.hpp"

#include <cmath>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <util/Logger.hpp>
#include <util/Math.hpp>

#include "game/BombPuzzleEntity.hpp"
#include "game/EnemyEntity.hpp"
#include "game/Player.hpp"
#include "game/Room.hpp"

BombEntity::BombEntity( Room& room, Player& player, sf::Vector2f pos )
   : Entity::Entity( room, player ),
     blown( false ),
     delay( 90 ),
     boom( 10 )
{
	delay += ( rand() % 30 ) - 15;
	spr.setPosition( sf::Vector2f( pos ) );
	spr.setScale( 0.25f, 0.25f );
	spr.setTextureRect( sf::IntRect( 0, 0, 8, 8 ) );
}

void BombEntity::update()
{
	if ( blown )
	{
		if ( boom >= 0 )
		{
			--boom;
			spr.setTextureRect( sf::IntRect( 0, 8, 16, 16 ) );
			spr.setOrigin( 8, 8 );
			spr.setScale( 0.65f, 0.65f );
		}
		
		return;
	}
	
	int frame = ( delay / 15 ) % 2;
	spr.setTextureRect( sf::IntRect( frame * 8, 0, 8, 8 ) );
	
	if ( delay >= 0 )
	{
		--delay;
		
		if ( delay < 0 )
		{
			blown = true;
			player.playSound( "res/sound/boom.wav" );
			for ( auto it = room.getEntities().begin(); it != room.getEntities().end(); ++it )
			{
				Entity* entity = it->get();
				if ( EnemyEntity* enemy = dynamic_cast< EnemyEntity* >( entity ) )
				{
					float dist = util::distance( enemy->getPosition(), spr.getPosition() );
					
					int dmg = 10 - ( static_cast< int >( dist ) * 2 );
					if ( dmg > 0 )
					{
						enemy->hurt( dmg );
					}
				}
				else if ( BombPuzzleEntity* bombPuzzle = dynamic_cast< BombPuzzleEntity* >( entity ) )
				{
					float dist = util::distance( bombPuzzle->getPosition(), spr.getPosition() );
					if ( dist < 5 )
					{
						bombPuzzle->breakWall();
					}
				}
			}
			
			float dist = util::distance( player.getPosition(), spr.getPosition() );
			
			int dmg = 10 - ( static_cast< int >( dist ) * 2 );
			if ( dmg > 0 )
			{
				player.hurt( dmg );
			}
		}
		
		return;
	}
	
	Entity::update();
}

void BombEntity::render( sf::RenderWindow& window )
{
	if ( !blown or boom >= 0 )
	{
		Entity::render( window );
	}
}

sf::FloatRect BombEntity::getCollisionRect()
{
	sf::FloatRect box;
	box.left = -( spr.getScale().x * ( spr.getTexture()->getSize().x / 2 ) );
	box.top = -( spr.getScale().y * ( spr.getTexture()->getSize().x / 2 ) );
	box.width = std::abs( box.left * 2 );
	box.height = std::abs( box.top * 2 );
	
	box.left += spr.getPosition().x + ( spr.getOrigin().x * spr.getScale().x );
	box.top += spr.getPosition().y + ( spr.getOrigin().x * spr.getScale().x );
	
	return box;
}

void BombEntity::setTexture( const sf::Texture& tex )
{
	spr.setTexture( tex, true );
	spr.setOrigin( 4, 4 );
}

