#include "game/Entity.hpp"

#include <SFML/Graphics/RenderWindow.hpp>
#include <util/Logger.hpp>
#include <util/Math.hpp>

#include "game/Player.hpp"
#include "game/Room.hpp"

Entity::Entity( Room& theRoom, Player& thePlayer )
   : room( theRoom ),
     player( thePlayer ),
     
     vel( 0.f, 0.f )
{
}

Entity::~Entity()
{
}

void Entity::update()
{
	// Solids
	for ( std::vector< sf::IntRect >::const_iterator it = room.getSolids().begin(); it != room.getSolids().end(); ++it )
	{
		const sf::IntRect& solid = ( * it );
		if ( solid.contains( spr.getPosition().x + vel.x, spr.getPosition().y + vel.y ) )
		{
			// +X
			if ( spr.getPosition().x <= solid.left and spr.getPosition().x + vel.x > solid.left )
			{
				vel.x = solid.left - spr.getPosition().x;
			}
			
			// -X
			if ( spr.getPosition().x >= solid.left + solid.width and spr.getPosition().x + vel.x < solid.left + solid.width )
			{
				vel.x = ( solid.left + solid.width ) - spr.getPosition().x;
			}
			
			// +Y
			if ( spr.getPosition().y <= solid.top and spr.getPosition().y + vel.y > solid.top )
			{
				vel.y = solid.top - spr.getPosition().y;
			}
			
			// -Y
			if ( spr.getPosition().y >= solid.top + solid.height and spr.getPosition().y + vel.y < solid.top + solid.height )
			{
				vel.y = ( solid.top + solid.height ) - spr.getPosition().y;
			}
		}
	}
	
	// Boundaries
	// Hack for end of the game
	if ( player.getArea() != "tower-top" )
	{
		if ( spr.getPosition().x < room.getBounds().left )
		{
			spr.setPosition( room.getBounds().left, spr.getPosition().y );
		}
		else if ( spr.getPosition().x > room.getBounds().left + room.getBounds().width )
		{
			spr.setPosition( room.getBounds().left + room.getBounds().width, spr.getPosition().y );
		}
		if ( spr.getPosition().y < room.getBounds().top )
		{
			spr.setPosition( spr.getPosition().x, room.getBounds().top );
		}
		else if ( spr.getPosition().y > room.getBounds().top + room.getBounds().height )
		{
			spr.setPosition( spr.getPosition().x, room.getBounds().top + room.getBounds().height );
		}
	}
	
	     if ( vel.x >=  0.01f ) vel.x -= 0.01f;
	else if ( vel.x <= -0.01f ) vel.x += 0.01f;
	else                        vel.x  = 0.00f;
	
	     if ( vel.y >=  0.01f ) vel.y -= 0.01f;
	else if ( vel.y <= -0.01f ) vel.y += 0.01f;
	else                        vel.y  = 0.00f;
	
	spr.move( vel );
}

void Entity::render( sf::RenderWindow& window )
{
	window.draw( spr );
}

sf::Vector2f Entity::getPosition() const
{
	return spr.getPosition();
}

sf::FloatRect Entity::getCollisionRect() const
{
	return spr.getGlobalBounds();
}

void Entity::addVelocity( float x, float y )
{
	vel.x += x;
	vel.y += y;
}

void Entity::push( float force, float dir )
{
	float dirRad = dir;//util::degreesToRadians( spr.getRotation() );
	
	vel.x += std::sin( dirRad ) * force;
	vel.y += std::cos( dirRad ) * force;
}
