#include "game/ItemEntity.hpp"

#include <cmath>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <util/Logger.hpp>
#include <util/Math.hpp>

#include "game/Player.hpp"

ItemEntity::ItemEntity( Room& room, Player& player, const std::string& theItem, sf::Vector2f pos )
   : Entity::Entity( room, player ),
     item( theItem ),
     collected( false ),
     delay( 50 )
{
	player.playSound( "res/sound/drop.wav" );
	spr.setPosition( sf::Vector2f( pos ) );
	spr.setScale( 0.25f, 0.25f );
	
	if ( item == "kitty" )
	{
		spr.setScale( 0.125f, 0.125f );
	}
}

void ItemEntity::update()
{
	if ( collected )
	{
		return;
	}
	
	if ( delay >= 0 )
	{
		--delay;
		return;
	}
	
	if ( getCollisionRect().intersects( player.getCollisionRect() ) )
	{
		for ( int i = 0; i < 5; ++i )
		{
			std::string& inv = player.getItems()[ i ];
			if ( inv == "" )
			{
				inv = item;
				collected = true;
				break;
			}
		}
	}
	
	Entity::update();
}

void ItemEntity::render( sf::RenderWindow& window )
{
	if ( !collected )
	{
		Entity::render( window );
	}
}

sf::FloatRect ItemEntity::getCollisionRect()
{
	sf::FloatRect box;
	box.left = -( spr.getScale().x * ( spr.getTexture()->getSize().x / 2 ) );
	box.top = -( spr.getScale().y * ( spr.getTexture()->getSize().x / 2 ) );
	box.width = std::abs( box.left * 2 );
	box.height = std::abs( box.top * 2 );
	
	box.left += spr.getPosition().x + ( spr.getOrigin().x * spr.getScale().x );
	box.top += spr.getPosition().y + ( spr.getOrigin().x * spr.getScale().x );
	
	return box;
}

void ItemEntity::setTexture( const sf::Texture& tex )
{
	spr.setTexture( tex, true );
	spr.setOrigin( tex.getSize().x / 2, tex.getSize().y / 2 );
}
