#include "game/ItemPuzzleEntity.hpp"

#include <memory>
#include <SFML/Graphics/Texture.hpp>
#include <util/Logger.hpp>

#include "game/ItemEntity.hpp"
#include "game/Player.hpp"
#include "game/Room.hpp"

ItemPuzzleEntity::ItemPuzzleEntity( Room& room, Player& player, sf::Vector2i pos )
   : Entity::Entity( room, player ),
     dropped( false )
{
	spr.setPosition( sf::Vector2f( pos ) );
}

void ItemPuzzleEntity::update()
{
	if ( dropped )
	{
		return;
	}

	for ( std::vector< std::shared_ptr< Entity > >::iterator it = room.getEntities().begin(); it != room.getEntities().end(); ++it )
	{
		Entity* entity = it->get();
		if ( ItemEntity* item = dynamic_cast< ItemEntity* >( entity ) )
		{
			if ( item->getCollisionRect().intersects( getCollisionRect() ) )
			{
				util::log( "Detected item, dropping key.\n" );
				room.addItem( "key_dungeon2", spr.getPosition() );
				dropped = true;
				player.playSound( "res/sound/solved.wav" );
				break;
			}
		}
	}
}

void ItemPuzzleEntity::render( sf::RenderWindow& window )
{
	if ( !dropped )
	{
		Entity::render( window );
	}
}

void ItemPuzzleEntity::setTexture( const sf::Texture& tex )
{
	spr.setTexture( tex, true );
	spr.setOrigin( tex.getSize().x / 2, tex.getSize().y / 2 );
}
