#ifndef PLAYER_HPP
#define PLAYER_HPP

#include <SFML/Graphics/Sprite.hpp>
#include <SFML/System/Vector2.hpp>
#include <string>
#include <vector>

namespace sf
{
	class Event;
	class RenderWindow;
}

class Game;
class GameScene;
class InputBinding;

class Player
{
	public:
		Player( Game& theGame, GameScene& theScene );

		void update();
		void render( sf::RenderWindow& window );

		bool isHardMode() const;

		const std::string& getArea() const;
		sf::Vector2i getRoom() const;
		sf::Vector2f getPosition() const;
		sf::FloatRect getCollisionRect() const;
		void moveTo( const std::string& theArea, sf::Vector2i theRoom, sf::Vector2f pos );

		int getHealth() const;
		int getHealthMax() const;
		bool hasPetKitty() const;
		bool didKittyHeal() const;

		void hurt( int amt );
		void kittyHeal();
		void push( float force, float dir );

		bool hasItem( const std::string& str ) const;
		std::vector< std::string>& getItems();
		const std::vector< std::string>& getItems() const;

		void setSelectedItem( unsigned int theSelected );
		unsigned int getSelectedItem() const;

		int getUseTime() const;
		int getStartingUseTime() const;

		void playSound( const std::string& path );

	private:
		Game& game;
		GameScene& scene;

		// Not a good way of doing this, but I'm just updating this thing. :P
		// I'll do it better in new projects.
		InputBinding& up;
		InputBinding& down;
		InputBinding& left;
		InputBinding& right;
		InputBinding& action;
		InputBinding& drop;
		InputBinding& slot1;
		InputBinding& slot2;
		InputBinding& slot3;
		InputBinding& slot4;
		InputBinding& slot5;
		InputBinding& prevItem;
		InputBinding& nextItem;

		std::string area;
		sf::Vector2i room;

		sf::Sprite spr;
		sf::Vector2f vel;
		sf::Vector2f forceVel;

		int health;
		int healthMax;
		int invulnTime;

		std::vector< std::string > items;
		unsigned int selected;
		int useTime;
		int startUseTime;

		bool hasKitty;
		bool kittyDidHeal;
		sf::Sprite kittySpr;

		void useItem( const std::string& item );
};

#endif // PLAYER_HPP
