#include "game/Room.hpp"

#include <util/Logger.hpp>

#include "Game.hpp"
#include "game/Area.hpp"
#include "game/BombEntity.hpp"
#include "game/BombPuzzleEntity.hpp"
#include "game/EnemyEntity.hpp"
#include "game/ItemEntity.hpp"
#include "game/ItemPuzzleEntity.hpp"
#include "game/SingleItemEntity.hpp"
#include "LoaderUtils.hpp"

Room::Room( Area& theArea, sf::Vector2i thePos )
   : area( theArea ),
     pos( thePos ),

	 bounds( 0, 0, 30, 30 ),
	 exitSides( true )
{
}

sf::Vector2i Room::getPosition() const
{
	return pos;
}

void Room::addBox( sf::IntRect collision, std::string func, std::vector< std::string > params )
{
	if ( func == "goto" or func == "gotolocked" or func == "gotobossexit" )
	{
		Exit exit;
		exit.collision = collision;
		exit.area = params[ 0 ];
		exit.room = LoaderUtils::parseVec2( params[ 1 ] ).data;
		exit.pos = LoaderUtils::parseVec2( params[ 2 ] ).data;
		exit.giveHealth = false;

		if ( func == "gotolocked" )
		{
			exit.itemReq = params[ 3 ];
		}
		else if ( func == "gotobossexit" )
		{
			exit.giveHealth = true;
		}

		exits.push_back( exit );
	}
	else if ( func == "get" )
	{
		Chest chest;
		chest.collision = collision;
		chest.item = params[ 0 ];
		if ( params[ 0 ] == "key" )
		{
			chest.item += "_" + params[ 1 ];
		}

		chests.push_back( chest );
	}
	else if ( func == "bounds" )
	{
		if ( params[ 0 ] == "boxed" )
		{
			exitSides = false;
			bounds = collision;
		}
	}
	else if ( func == "solid" )
	{
		solids.push_back( collision );
	}
	else if ( func == "spawn" )
	{
		int amount = util::fromString< int >( params[ 1 ] );
		for ( int i = 0; i < amount; ++i )
		{
			sf::Vector2i pos( collision.left, collision.top );
			pos.x += rand() % collision.width;
			pos.y += rand() % collision.height;

			std::shared_ptr< Entity > entity;
			if ( params[ 0 ] == "caveMonster" or params[ 0 ] == "dungeonMonster" or params[ 0 ] == "towerMonster" )
			{
				std::shared_ptr< EnemyEntity > enemy( new EnemyEntity( ( * this ), area.getPlayer(), params[ 0 ], pos ) );

				enemy->setHealth( 4 );
				enemy->setStrength( ( params[ 0 ] == "towerMonster" ) ? 3 : ( ( params[ 0 ] == "dungeonMonster" ) ? 2 : 1 ) );
				enemy->setSpeed( 0.1f + ( ( ( rand() % 11 ) / 20.f ) / 20 ) );
				enemy->setTexture( * area.getGame().getTexture( "res/entities/" + std::string( ( params[ 0 ] == "caveMonster" ) ? "cave" : ( ( params[ 0 ] == "dungeonMonster" ) ? "dungeon" : "tower" ) ) + "-monster.png" ) );

				entity = enemy;
			}
			else if ( params[ 0 ] == "caveBoss" )
			{
				std::shared_ptr< EnemyEntity > enemy( new EnemyEntity( ( * this ), area.getPlayer(), params[ 0 ], pos ) );

				enemy->setHealth( 12 );
				enemy->setStrength( 2 );
				enemy->setSpeed( 0.1f );
				enemy->setTexture( * area.getGame().getTexture( "res/entities/cave-boss.png" ) );

				entity = enemy;
			}
			else if ( params[ 0 ] == "dungeonBoss" )
			{
				std::shared_ptr< EnemyEntity > enemy( new EnemyEntity( ( * this ), area.getPlayer(), params[ 0 ], pos ) );

				enemy->setHealth( 40 );
				enemy->setStrength( 3 );
				enemy->setSpeed( 0.05f );
				enemy->setTexture( * area.getGame().getTexture( "res/entities/dungeon-boss.png" ) );

				entity = enemy;
			}
			else if ( params[ 0 ] == "towerBoss" )
			{
				std::shared_ptr< EnemyEntity > enemy( new EnemyEntity( ( * this ), area.getPlayer(), params[ 0 ], pos ) );

				enemy->setHealth( 128 );
				enemy->setStrength( 4 );
				enemy->setSpeed( 0.f );
				enemy->setTexture( * area.getGame().getTexture( "res/entities/tower-boss.png" ) );

				entity = enemy;
			}
			else if ( params[ 0 ] == "itemPuzzle" )
			{
				std::shared_ptr< ItemPuzzleEntity > itemPuzzle( new ItemPuzzleEntity( ( * this ), area.getPlayer(), pos ) );

				itemPuzzle->setTexture( * area.getGame().getTexture( "res/entities/item-puzzle.png" ) );

				entity = itemPuzzle;
			}
			else if ( params[ 0 ].length() > 5 and params[ 0 ].substr( 0, 5 ) == "item_" )
			{
				addItem( params[ 0 ].substr( 5 ), sf::Vector2f( pos ) );
			}
			else if ( params[ 0 ] == "bombPuzzle" )
			{
				std::shared_ptr< BombPuzzleEntity > bombPuzzle( new BombPuzzleEntity( ( * this ), area.getPlayer(), pos ) );

				bombPuzzle->setTexture( * area.getGame().getTexture( "res/entities/bomb-puzzle.png" ) );

				entity = bombPuzzle;
			}
			else if ( params[ 0 ] == "singleItemEnforcer" )
			{
				std::shared_ptr< SingleItemEntity > singleItem( new SingleItemEntity( ( * this ), area.getPlayer(), pos ) );

				singleItem->setTexture( * area.getGame().getTexture( "res/gui/limit.png" ) );

				entity = singleItem;
			}
			else
			{
				util::log( "\t\tDon't know how to spawn \"%s\"!\n", params[ 0 ] );
			}

			if ( entity )
			{
				entities.push_back( entity );
			}
		}
	}
	else
	{
		util::log( "\t\tDon't know how to \"%s\"!\n", func );
	}
}

sf::IntRect Room::getBounds() const
{
	return bounds;
}

bool Room::canExitSides() const
{
	return exitSides;
}

std::vector< Room::Exit >& Room::getExits()
{
	return exits;
}

const std::vector< Room::Exit >& Room::getExits() const
{
	return exits;
}

std::vector< Room::Chest >& Room::getChests()
{
	return chests;
}

const std::vector< Room::Chest >& Room::getChests() const
{
	return chests;
}

std::vector< sf::IntRect >& Room::getSolids()
{
	return solids;
}

const std::vector< sf::IntRect >& Room::getSolids() const
{
	return solids;
}

std::vector< std::shared_ptr< Entity > >& Room::getEntities()
{
	return entities;
}

const std::vector< std::shared_ptr< Entity > >& Room::getEntities() const
{
	return entities;
}

void Room::addItem( const std::string& name, sf::Vector2f pos )
{
	std::shared_ptr< ItemEntity > item = std::make_shared< ItemEntity >( ( * this ), area.getPlayer(), name, pos );
	area.getGame().getTexture( "res/items/" + name + ".png" );
	item->setTexture( * area.getGame().getTexture( "res/items/" + name + ".png" ) );
	entities.push_back( std::static_pointer_cast< Entity >( item ) );
}

void Room::spawnEnemy( const std::string& type, sf::Vector2f pos )
{
	std::shared_ptr< Entity > entity;
	if ( type == "caveMonster" or type == "dungeonMonster" or type == "towerMonster" )
	{
		std::shared_ptr< EnemyEntity > enemy( new EnemyEntity( ( * this ), area.getPlayer(), type, sf::Vector2i( pos ) ) );

		enemy->setHealth( 4 );
		enemy->setStrength( ( type == "towerMonster" ) ? 3 : ( ( type == "dungeonMonster" ) ? 2 : 1 ) );
		enemy->setSpeed( 0.1f + ( ( ( rand() % 11 ) / 20.f ) / 25 ) );
		enemy->setTexture( * area.getGame().getTexture( "res/entities/" + std::string( ( type == "caveMonster" ) ? "cave" : ( ( type == "dungeonMonster" ) ? "dungeon" : "tower" ) ) + "-monster.png" ) );

		entity = enemy;
	}

	if ( entity )
	{
		entities.push_back( entity );
	}
}

void Room::makeBombAt( sf::Vector2f pos )
{
	std::shared_ptr< BombEntity > bomb = std::make_shared< BombEntity >( ( * this ), area.getPlayer(), pos );
	bomb->setTexture( * area.getGame().getTexture( "res/entities/bomb.png" ) );
	entities.push_back( bomb );
}

void Room::resetEnemies()
{
	for ( auto it = entities.begin(); it != entities.end(); ++it )
	{
		if ( EnemyEntity* enemy = dynamic_cast< EnemyEntity* >( it->get() ) )
		{
			enemy->reset();
		}
	}
}
