#ifndef ROOM_HPP
#define ROOM_HPP

#include <memory>
#include <SFML/Graphics/Rect.hpp>
#include <SFML/System/Vector2.hpp>
#include <string>
#include <vector>

#include "game/Entity.hpp"

class Area;

class Room
{
	public:
		Room( Area& theArea, sf::Vector2i thePos );

		struct Exit
		{
			sf::IntRect collision;

			std::string area;
			sf::Vector2i room;
			sf::Vector2i pos;

			std::string itemReq;
			bool giveHealth;
		};

		struct Chest
		{
			sf::IntRect collision;
			std::string item;
		};

		sf::Vector2i getPosition() const;

		void addBox( sf::IntRect collision, std::string func, std::vector< std::string > params );

		sf::IntRect getBounds() const;
		bool canExitSides() const;

		std::vector< Exit >& getExits();
		const std::vector< Exit >& getExits() const;

		std::vector< Chest >& getChests();
		const std::vector< Chest >& getChests() const;

		std::vector< sf::IntRect >& getSolids();
		const std::vector< sf::IntRect >& getSolids() const;

		std::vector< std::shared_ptr< Entity > >& getEntities();
		const std::vector< std::shared_ptr< Entity > >& getEntities() const;

		void addItem( const std::string& name, sf::Vector2f pos );
		void spawnEnemy( const std::string& type, sf::Vector2f pos );
		void makeBombAt( sf::Vector2f pos );
		void resetEnemies();

	private:
		Area& area;
		const sf::Vector2i pos;

		sf::IntRect bounds;
		bool exitSides;
		std::vector< Exit > exits;
		std::vector< Chest > chests;
		std::vector< sf::IntRect > solids;
		std::vector< std::shared_ptr< Entity > > entities;
};

#endif // ROOM_HPP
