#include "game/SingleItemEntity.hpp"

#include <SFML/Graphics/Texture.hpp>
#include <util/Logger.hpp>

#include "game/Player.hpp"
#include "game/Room.hpp"

SingleItemEntity::SingleItemEntity( Room& room, Player& player, sf::Vector2i pos )
   : Entity::Entity( room, player )
{
	//*
	exitIndex = room.getExits().size();
	room.getExits().push_back( Room::Exit() );
	Room::Exit& exit = room.getExits()[ exitIndex ];
	
	exit.collision = sf::IntRect( pos.x, pos.y, 6, 1 );
	
	exit.area = "tower";
	exit.room = sf::Vector2i( 0, -3 );
	exit.pos = sf::Vector2i( 15, 28 );
	
	exit.itemReq = "unobtainable";
	exit.giveHealth = false;
	//*/
}

void SingleItemEntity::update()
{
	int weaponCount = 0;
	for ( std::size_t i = 0; i < player.getItems().size(); ++i )
	{
		std::string item = player.getItems()[ i ];
		if ( item == "dagger" or item == "leaf" or item == "bomb" or item == "potato" )
		{
			++weaponCount;
		}
	}
	
	room.getExits()[ exitIndex ].itemReq = ( weaponCount > 1 ) ? "unobtainable" : "";
}

void SingleItemEntity::render( sf::RenderWindow& window )
{
	if ( room.getExits()[ exitIndex ].itemReq != "" )
	{
		Entity::render( window );
	}
}

void SingleItemEntity::setTexture( const sf::Texture& tex )
{
	float scale = 25.f / tex.getSize().x;
	
	spr.setTexture( tex, true );
	spr.setScale( scale, scale );
	spr.setPosition( ( 30 - spr.getGlobalBounds().width ) / 2, ( 30 - spr.getGlobalBounds().height ) / 2 );
}

