#ifndef GAMESCENE_HPP
#define GAMESCENE_HPP

#include <list>
#include <map>
#include <SFML/Audio/Music.hpp>
#include <SFML/Audio/Sound.hpp>
#include <SFML/Config.hpp>

#include "game/Area.hpp"
#include "game/Player.hpp"
#include "Scene.hpp"

class GameScene : public Scene
{
	public:
		GameScene( Game& theGame, const SceneChangeEvent& event );
		
		virtual void update();
		virtual void update( const sf::Event& event );
		virtual void render( sf::RenderWindow& window );
		
		Area& getArea( const std::string& name );
		const Area& getArea( const std::string& name ) const;
		
		Player& getPlayer();
		const Player& getPlayer() const;
		
		void playSound( const std::string& path );
		
		bool isHardMode() const;
	
	private:
		std::map< std::string, std::shared_ptr< Area > > areas;
		Player player;
		std::list< sf::Sound > sounds;
		sf::Music music;
		std::string currMusic;
		bool paused;
		
		const bool hard;
		
		bool loadingWorld;
		
		void loadWorld();
		std::string getTargetMusic() const;
};

#endif // GAMESCENE_HPP
