#include "scenes/MainMenuScene.hpp"

#include <SFML/Window/Event.hpp>

#include "Game.hpp"
#include "scenes/GameScene.hpp"
#include "scenes/OptionsScene.hpp"
#include "Utility.hpp"

MainMenuScene::MainMenuScene( Game& theGame, const SceneChangeEvent& event )
   : Scene::Scene( theGame ),
     font( * theGame.getFont( "res/ken_fonts/kenpixel.ttf" ) )
{
	title.setString( "One Chance" );
	title.setFont( font );
	title.setCharacterSize( 60 );
	title.setColor( sf::Color::White );
	title.setPosition( ( game.getWindow().getSize().x - title.getLocalBounds().width ) / 2, 25 );
	
	easy.setTexture( * game.getTexture( "res/gui/easy.png" ) );
	easy.setOrigin( easy.getLocalBounds().width / 2, easy.getLocalBounds().height / 2 );
	easy.setPosition( game.getWindow().getSize().x / 4, game.getWindow().getSize().y / 2 );
	easy.setScale( 5, 5 );
	
	hard = easy;
	hard.setTexture( * game.getTexture( game.options.hardUnlocked ? "res/gui/hard.png" : "res/gui/locked.png" ) );
	hard.move( game.getWindow().getSize().x / 2, 0 );
	
	options = title;
	options.setString( "Options" );
	options.setCharacterSize( 35 );
	options.setPosition( ( game.getWindow().getSize().x - options.getLocalBounds().width ) / 2, game.getWindow().getSize().y - 35 - 50 );
}

void MainMenuScene::update()
{
}

void MainMenuScene::update( const sf::Event& event )
{
	if ( event.type == sf::Event::MouseButtonPressed )
	{
		if ( event.mouseButton.button == sf::Mouse::Left )
		{
			if ( easy.getGlobalBounds().contains( event.mouseButton.x, event.mouseButton.y ) )
			{
				game.changeScenes< GameScene >( SceneChangeEvent( SceneChangeEvent::PlayEasy ) );
			}
			else if ( game.options.hardUnlocked and hard.getGlobalBounds().contains( event.mouseButton.x, event.mouseButton.y ) )
			{
				game.changeScenes< GameScene >( SceneChangeEvent( SceneChangeEvent::PlayHard ) );
			}
			else if ( options.getGlobalBounds().contains( event.mouseButton.x, event.mouseButton.y ) )
			{
				game.changeScenes< OptionsScene >();
			}
		}
	}
}

void MainMenuScene::render( sf::RenderWindow& window )
{
	window.draw( title );
	
	window.draw( easy );
	window.draw( hard );
	
	handleHighlight( window, options );
	window.draw( options );
}
